%  This program create a VTK file with the geometry of a mask array
%  The resulting y axis in paraview is the rightward direction in the original
%  image, and the x axis is the downward direction of the original image

%  It then assigns the 3-component velocity information in the 
%  DICOM images into the VTK.

%  4/6/2012  Ethan Kung  keo@alumni.stanford.edu


%%  Need to uncomment this section if using this file as stand-alone
% input= '1';  %input filename
% output= 'output.vtk';  %output filename
% Venc=15;   %velocity encoding value
% 
% info= dicominfo('MRIm02');  %input filename
% mask=roipoly(dicomread('MRIm02'));   %input filename
%%

%phase=x velocity, phase2=y velocity, phase3=z velocity
phase=dicomread(input);
phase =double(phase)/32768 * Venc_in;
phase = phase .* double(mask);  %clean out noise when calculating the velocities of edge nodes requiring averaging adjacent cells.

phase2=dicomread(input2);
phase2 =double(phase2)/32768 * Venc_in;
phase2 = phase2 .* double(mask); 

phase3=dicomread(input3);
phase3 =double(phase3)/32768 * Venc_thru;
phase3 = phase3 .* double(mask); 

lr= length(mask(:,1));  %length of rows
lc= length(mask(1,:));  %length of columes

%unwrap through plane velocity if aliased
for i=2:lr
    for j=1:lc
        if mask(i,j)==1 && abs(phase3(i,j)-phase3(i-1,j)) >= 1.2*Venc_thru
            phase3(i,j)= phase3(i,j) - abs(phase3(i,j))/phase3(i,j)* 2*Venc_thru ;
        %elseif mask(i,j)==1 && -(phase3(i,j)-phase3(i-1,j)) >= 1.3*Venc_thru
        %    phase3(i,j)= phase3(i,j) +  2*Venc_thru ;
        end
    end
end

%% This constructs an array which keeps track of the points
% adjacent to the "masked" pixels
% Points 3rd dimension values: 1 mask flag, 2 point number, 3 velocity x, 4 velocity y, 5 velocity z  
points=zeros( lr+1, lc+1, 5);
for i=1:lr
    for j=1:lc
        if mask(i,j)==1
        points(i,j,1)=1;
        points(i+1,j,1)=1;
        points(i,j+1,1)=1;
        points(i+1,j+1,1)=1;
        end
    end
end

%% This numbers the "existing" points in the points array
% Assigns a velocity value to the "existing" points
% Also stores the point coordinates in "vtkpoints" array, which will be
% used as the points table in the vtk file
vtkpoints=zeros(7000, 2);
n=0;        %keeping track of number of points
for i=1:lr+1
    for j=1:lc+1
        if points(i,j,1)==1
        points(i,j,2)= n;   %numbers the points
        points(i,j,3)= (phase(i,j)+phase(i-1,j)+phase(i,j-1)+phase(i-1,j-1))/4;  %calculate velocity x
        points(i,j,4)= (phase2(i,j)+phase2(i-1,j)+phase2(i,j-1)+phase2(i-1,j-1))/4;  %calculate velocity y
        points(i,j,5)= (phase3(i,j)+phase3(i-1,j)+phase3(i,j-1)+phase3(i-1,j-1))/4;  %calculate velocity z
        n=n+1;
        vtkpoints(n,1)=i;
        vtkpoints(n,2)=j;
        end
    end
end
final_vtkpoints=zeros(n, 3);
final_vtkpoints(:,1:2)=vtkpoints(1:n, :);  %this is the final points array
final_vtkpoints(:,1)=final_vtkpoints(:,1) * info.PixelSpacing(1)/10;
final_vtkpoints(:,2)=final_vtkpoints(:,2) * info.PixelSpacing(2)/10;  %scale to pixel spacing


%% this constructs the table for the vtk file to list the cells
% values in the variable "cells" are directly used to construct VTK file
% Thus do not modify
cells=zeros(n, 5);
m=1;        %keeping track of number of cells
for i=1:lr
    for j=1:lc
        if mask(i,j)==1
        cells(m,2)= points(i,j,2);
        cells(m,3)= points(i+1,j,2);
        cells(m,4)= points(i+1,j+1,2);
        cells(m,5)= points(i,j+1,2);
        m=m+1;
        end
    end
end
final_cells=cells(1:m-1, :);
final_cells(:,1)=4;


% This constructs the "cell type" section of the VTK file
dummy=zeros(m-1);
dummy=dummy(:,1);
dummy(:)=5;


%% Write the output VTK file
fid=fopen(output,'w');
fprintf(fid, '# vtk DataFile Version 3.0\n2D MR Data\nASCII\n\nDATASET POLYDATA\nPOINTS ');
fprintf(fid, '%u', n);
fprintf(fid, ' float\n');
fprintf(fid, '%.4f %.4f %.4f\n', transpose(final_vtkpoints));

fprintf(fid, '\nPOLYGONS ');
fprintf(fid, '%u ', m-1);
fprintf(fid, '%u \n', (m-1)*5);
fprintf(fid, '%u %u %u %u %u\n', transpose(final_cells));

% fprintf(fid, '\nCELL_TYPES ');
% fprintf(fid, '%u \n', m-1);
% fprintf(fid, '%u\n', dummy);

fprintf(fid, '\nCELL_DATA ');
fprintf(fid, '%u\n', m-1);
fprintf(fid, 'SCALARS velocity_mag float\nLOOKUP_TABLE default\n');
for i=1:lr
    for j=1:lc
        if mask(i,j)==1
        fprintf(fid, '%.4f\n', sqrt(phase(i,j)^2+ phase2(i,j)^2+ phase3(i,j)^2)  );
        end
    end
end

flowrate= sum(sum(phase3)) * info.PixelSpacing(1)/10 * info.PixelSpacing(2)/10;
dlmwrite('1flowrate',-flowrate, '-append');



%% This write the 3-component velocity information into the VTK file
fprintf(fid, '\nPOINT_DATA ');
fprintf(fid, '%u\n', n);
fprintf(fid, '\nVECTORS velocity float\n');
for i=1:lr+1
    for j=1:lc+1
        if points(i,j,1)==1
        %fprintf(fid, '%.4f %.4f %.4f \n', -points(i,j,3), -points(i,j,4), -points(i,j,5)  ); %original default orientation
        fprintf(fid, '%.4f %.4f %.4f \n', -points(i,j,4), -points(i,j,3), -points(i,j,5)  ); %orientation for S2
        %fprintf(fid, '0 0 %.4f \n', -points(i,j,5)  );
        end
    end
end

