%  This program create a VTK file with the geometry of a mask array
%  The resulting y axis in paraview is the rightward direction in the original
%  image, and the x axis is the downward direction of the original image
%  It then assigns the velocity information in the DICOM image into the VTK
%  2/14/2012  Ethan Kung  keo@alumni.stanford.edu


%%  Need to uncomment this section if using this file as stand-alone
% input= '1';  %input file name
% output= 'output.vtk';   %output filename
% Venc=15;   %velocity encoding value
% 
% info= dicominfo('MRIm02');  %input filename
% mask=roipoly(dicomread('MRIm02'));
%%

phase=dicomread(input);
phase =double(phase)/32768 * Venc;
phase = phase .* double(mask);  %clean out noise when calculating the velocities of edge nodes requiring averaging adjacent cells.
lr= length(mask(:,1));  %length of rows
lc= length(mask(1,:));  %length of columes


%% This constructs an array which keeps track of the points
% adjacent to the "masked" pixels
% Points 3rd dimension values: 1 mask flag, 2 point number, 3 velocity Z  
points=zeros( lr+1, lc+1, 3);
for i=1:lr
    for j=1:lc
        if mask(i,j)==1
        points(i,j,1)=1;
        points(i+1,j,1)=1;
        points(i,j+1,1)=1;
        points(i+1,j+1,1)=1;
        end
    end
end

%% This numbers the "existing" points in the points array
% Assigns a velocity value to the "existing" points
% Also stores the point coordinates in "vtkpoints" array, which will be
% used as the points table in the vtk file
vtkpoints=zeros(7000, 2);
n=0;        %keeping track of number of points
for i=1:lr+1
    for j=1:lc+1
        if points(i,j,1)==1
        points(i,j,2)= n;   %numbers the points
        points(i,j,3)= (phase(i,j)+phase(i-1,j)+phase(i,j-1)+phase(i-1,j-1))/4;  %calculate velocity
        n=n+1;
        vtkpoints(n,1)=i;
        vtkpoints(n,2)=j;
        end
    end
end
final_vtkpoints=zeros(n, 3);
final_vtkpoints(:,1:2)=vtkpoints(1:n, :);  %this is the final points array
final_vtkpoints(:,1)=final_vtkpoints(:,1) * info.PixelSpacing(1)/10;
final_vtkpoints(:,2)=final_vtkpoints(:,2) * info.PixelSpacing(2)/10;  %scale to pixel spacing


%% this constructs the table for the vtk file to list the cells
% values in the variable "cells" are directly used to construct VTK file
% Thus do not modify
cells=zeros(n, 5);
m=1;        %keeping track of number of cells
for i=1:lr
    for j=1:lc
        if mask(i,j)==1
        cells(m,2)= points(i,j,2);
        cells(m,3)= points(i+1,j,2);
        cells(m,4)= points(i+1,j+1,2);
        cells(m,5)= points(i,j+1,2);
        m=m+1;
        end
    end
end
final_cells=cells(1:m-1, :);
final_cells(:,1)=4;


% This constructs the "cell type" section of the VTK file
dummy=zeros(m-1);
dummy=dummy(:,1);
dummy(:)=5;


%% Write the output VTK file
fid=fopen(output,'w');
fprintf(fid, '# vtk DataFile Version 1.0\nTest 2D Grid\nASCII\n\nDATASET UNSTRUCTURED_GRID\nPOINTS ');
fprintf(fid, '%u', n);
fprintf(fid, ' float\n');
fprintf(fid, '%.4f %.4f %.4f\n', transpose(final_vtkpoints));

fprintf(fid, '\nCELLS ');
fprintf(fid, '%u ', m-1);
fprintf(fid, '%u \n', (m-1)*5);
fprintf(fid, '%u %u %u %u %u\n', transpose(final_cells));

fprintf(fid, '\nCELL_TYPES ');
fprintf(fid, '%u \n', m-1);
fprintf(fid, '%u\n', dummy);

fprintf(fid, '\nCELL_DATA ');
fprintf(fid, '%u\n', m-1);
fprintf(fid, 'SCALARS velocity_mag float\nLOOKUP_TABLE default\n');
for i=1:lr
    for j=1:lc
        if mask(i,j)==1
        fprintf(fid, '%.4f\n', phase(i,j));
        end
    end
end

flowrate= sum(sum(phase)) * info.PixelSpacing(1)/10 * info.PixelSpacing(2)/10;
dlmwrite('1flowrate',flowrate, '-append');



%% This write the 3-component velocity information into the VTK file
fprintf(fid, '\nPOINT_DATA ');
fprintf(fid, '%u\n', n);
fprintf(fid, '\nVECTORS velocity float\n');
for i=1:lr+1
    for j=1:lc+1
        if points(i,j,1)==1
        fprintf(fid, '0 0 %.4f\n', points(i,j,3));
        end
    end
end

